import { getComponent } from "./component_manager.js";
import Module from "./module.js";
import Node from "./node.js";
import observer from "./observer.js";
import Group from "./group.js";
import { Events } from "./events.js";
let patchIdCounter = 0;
class Patch {
    constructor(ownerNode) {
        this.id = ++patchIdCounter;
        this.nodes = {};
        this.ownerModuleNode = null;
        this.groups = {};
        this.ownerModuleNode = ownerNode;
    }
    isModule() {
        return this.ownerModuleNode != null;
    }
    createGroup(id, name, comment = "", nodeIdList = []) {
        let group = new Group(id, name || "New Group", comment, [...nodeIdList]);
        this.groups[group.id] = group;
        observer.triggerEvent(Events.OnCreateGroup, group);
        return group;
    }
    removeGroup(id) {
        let group = this.groups[id];
        delete this.groups[id];
        observer.triggerEvent(Events.OnRemoveGroup, id);
        return group;
    }
    createNode(id, componentIdentKey, properties, metadata) {
        if (getComponent(componentIdentKey) == null) {
            return null;
        }
        //let id = IDGenerator.next();
        let node;
        if (componentIdentKey == "core.module") {
            node = this.nodes[id] = new Module(this, id, componentIdentKey);
        }
        else {
            node = new Node(this, id, componentIdentKey);
            if (componentIdentKey == "core.input" || componentIdentKey == "core.output") {
                if (this.ownerModuleNode != null) {
                    if (componentIdentKey == "core.input") {
                        this.ownerModuleNode.inputNodes[id] = node;
                    }
                    else if (componentIdentKey == "core.output") {
                        this.ownerModuleNode.outputNodes[id] = node;
                    }
                }
                else {
                    return null;
                }
            }
            this.nodes[id] = node;
        }
        node.properties = properties || {};
        node.metadata = metadata || {};
        if (node != null) {
            observer.triggerEvent(Events.OnCreateNode, node);
        }
        return node;
    }
    removeNode(nodeId, removePorts = true) {
        let node = this.nodes[nodeId];
        let removedFromGroupId = null;
        let removedEmptyGroup = null;
        if (node != null) {
            if (removePorts) {
                for (let portKey of Object.keys(node.outputs)) {
                    node.removeOutputConnection(portKey, null);
                }
                for (let portKey of Object.keys(node.inputs)) {
                    node.removeInputConnection(portKey, null);
                }
            }
            for (let group of Object.values(this.groups)) {
                if (group.hasNodeId(nodeId)) {
                    group.removeNodeId(nodeId);
                    removedFromGroupId = group.id;
                    if (group.isEmpty()) {
                        this.removeGroup(group.id);
                        removedEmptyGroup = group;
                    }
                    break;
                }
            }
            delete this.nodes[nodeId];
            observer.triggerEvent(Events.OnRemoveNode, node);
        }
        return {
            removedFromGroupId: removedFromGroupId,
            removedEmptyGroup: removedEmptyGroup
        };
    }
    getNode(nodeId) {
        let node = this.nodes[nodeId];
        return node;
    }
    findConnection(connectionId) {
        for (let node of Object.values(this.nodes)) {
            for (let port of Object.values(node.outputs)) {
                for (let [id, connection] of port.connections) {
                    if (id == connectionId) {
                        return connection;
                    }
                }
            }
        }
        return null;
    }
    mapNode(node) {
        this.nodes[node.id] = node;
        observer.triggerEvent(Events.OnMoveNodeFromModule, node);
    }
}
export default Patch;
//# sourceMappingURL=patch.js.map