import { Events } from "./events.js";
import Node, { NodeType } from "./node.js";
import observer from "./observer.js";
import Patch, { NodeMap } from "./patch.js";

class Module extends Node {
    type = NodeType.Module;
    name: string = "New module";
    patch: Patch = new Patch(this);
    inputNodes: NodeMap = {};
    outputNodes: NodeMap = {};

    isConnectableInputPort(key: string) {
        return key in this.inputNodes;
    }

    isConnectableOutputPort(key: string) {
        return key in this.outputNodes;
    }

    getConnectableInputPorts() {
        return Object.entries(this.inputNodes).map(([id, node]) => {
            return [id, node.name];
        });
    }

    mapNode(node: Node) {
        this.patch.nodes[node.id] = node;
        observer.triggerEvent(Events.OnMoveNodeToModule, node);
    }
}

export default Module;